BingEst <- function(tau,x0,b,M,nObs,Nbig,Nsave,B,AMLEf)
{

# This function estimates the parameters of a q-dimensional Bingham random variable by means of:
# 1. the maximization of the log-likelihood obtained by estimating the normalization constant via the
# Holonomic Gradient Method (HGM)
# 2. the Approximate Maximum Likelihood Method
# See Bee, M., Benedetti, R. and Espa, G. (2015), 'Approximate likelihood inference for the Bingham distribution',
# DEM working paper 2015/02, for details.
#
# Input: 
# tau: observed value of the sufficient statistics
# x0: starting values of the parameters (only used for the HGM method)
# b: tuning constant to be used in the simulation of the Bingham distribution. Typically set to 1.
# M: constant to be used in the acceptance-rejection algorithm that simulated the Bingham distribution. It is 
# convenient to set M=sup{f/x)/g(x)}, where f(x) is the unnormalized Bingham density and g(x) is the 
# unnormalized angular central Gaussian density
# nObs: sample size of the observed data
# Nbig: number of simulated candidate parameter values (only used for the AMLE method)
# Nsave: ABC sample size
# B: (2 x (q-1)) matrix whose i-th column contains the upper and lower bound of the uniform distribution from
# which the candidate values of the i-th (i=1,...q-1) parameter are sampled.
# AMLEf: flag. If AMLEf=0, only HGM estimation is performed;
#              if AMLEf=1, only AMLE is performed
#              if AMLEf=2, estimation is performed with both methods.
#
# Output:
#
# a list containing:
#
# HGM: the estimators obtained via the HGM method
# ABCSample: the ABC sample used in the AMLE procedure
# AMLE: the estimators obtained via the AMLE method
#
# Usage: 
# tau <- c(.02,.4)
# x0 <- c(.23,.4)
# b <- 1
# M <- 1.92
# nObs <- 100
# Nbig <- 200000
# Nsave <- 1000
# B <- cbind(c(20,30),c(0,2))
# AMLEf <- 2
# fit <- BingEst(tau,x0,b,M,nObs,Nbig,Nsave,B,AMLEf)
#
# Authors: Marco Bee, Roberto Benedetti, Giuseppe Espa
# This version: August 2016.

# 1. HGM

require(hgm)
require(tcltk)
require(ks)
source('c:\\marco\\dropbox\\R\\rBing.r')
source('c:\\marco\\dropbox\\bingham\\programs\\BingLoglik.r')
source('c:\\marco\\dropbox\\bingham\\programs\\BingLoglikGrad.r')
source('c:\\marco\\dropbox\\R\\norm_vec_s.r')
qq = length(x0) + 1

# HGM only

if (AMLEf == 0)
{
  res = optim(x0,BingLoglik,gr=BingLoglikGrad, method = "L-BFGS-B", tau, nObs, control = list(trace = 4, fnscale = -2))
  HGMEst = -res$par
  HGMEst[HGMEst<0] = 0
  res = HGMEst
}

# AMLE only

if (AMLEf == 1)
{
  lambdaSim = matrix(0,Nbig,qq-1)
  ss = rep(0,Nbig)
  i = 1
  pb <- tkProgressBar("Progress bar", "Some information in %", 0, Nbig, 1)
  while (i<=Nbig)
  {
    for (j in 1:(qq-1))
    {
     lambdaSim[i,j] = runif(1,B[1,j],B[2,j])
    }
    lambdaSimvec = c(lambdaSim[i,],0)
    LambdaSim = diag(lambdaSimvec)
    Y_sim = rBing(nObs,b,LambdaSim,M)
    tau_sim = colMeans(Y_sim[,1:(qq-1)]^2)
    ss[i] = norm_vec_s(tau,tau_sim)
    info <- sprintf("%d%% done", round(i/(Nbig/100)))
    setTkProgressBar(pb, i, sprintf("test (%s)", info), info)
    i = i + 1
  }
  close(pb)
  matrice = cbind(lambdaSim, ss)
  last = ncol(matrice)
  ordine = order(matrice[,last], decreasing = F)
  ms = matrice[ordine,]
  ABCsam = ms[1:Nsave,1:(qq-1)]
  if (last < 8)
  {
     bw = Hscv(ABCsam)
     f_eps = kde(ABCsam,bw,eval.points=ABCsam)
     AMLE = ABCsam[which.max(f_eps$estimate),]
  }
  if (last >= 8)
  {
  AMLE = rep(0,qq-1)
  for (ss in 1:(qq-1))
  {
      vettore = ABCsam[,ss]
      bw = hscv(vettore)
      f_eps = kde(vettore,bw,eval.points=vettore)
      AMLE[ss] = vettore[which.max(f_eps$estimate)]
  }
  }
  res = list(ABCSample = ABCsam, AMLE = AMLE)
}

# 3. HGM and AMLE

if (AMLEf == 2)
{
  res = optim(x0,BingLoglik,gr=BingLoglikGrad, method = "L-BFGS-B", tau, nObs, control = list(trace = 4, fnscale = -2))
  HGMEst = -res$par
  HGMEst[HGMEst<0] = 0
  res = HGMEst
  lambdaSim = matrix(0,Nbig,qq-1)
  ss = rep(0,Nbig)
  i = 1
  pb <- tkProgressBar("Progress bar", "Some information in %", 0, Nbig, 1)
  while (i<=Nbig)
  {
    for (j in 1:(qq-1))
    {
     lambdaSim[i,j] = runif(1,B[1,j],B[2,j])
    }
    lambdaSimvec = c(lambdaSim[i,],0)
    LambdaSim = diag(lambdaSimvec)
    Y_sim = rBing(nObs,b,LambdaSim,M)
    tau_sim = colMeans(Y_sim[,1:(qq-1)]^2)
    ss[i] = norm_vec_s(tau,tau_sim)
    info <- sprintf("%d%% done", round(i/(Nbig/100)))
    setTkProgressBar(pb, i, sprintf("test (%s)", info), info)
    i = i + 1
  }
  close(pb)
  matrice = cbind(lambdaSim, ss)
  last = ncol(matrice)
  ordine = order(matrice[,last], decreasing = F)
  ms = matrice[ordine,]
  ABCsam = ms[1:Nsave,1:(qq-1)]
  if (last < 8)
  {
     bw = Hscv(ABCsam)
     f_eps = kde(ABCsam,bw,eval.points=ABCsam)
     AMLE = ABCsam[which.max(f_eps$estimate),]
  }
  if (last >= 8)
  {
  AMLE = rep(0,qq-1)
  for (ss in 1:(qq-1))
  {
      vettore = ABCsam[,ss]
      bw = hscv(vettore)
      f_eps = kde(vettore,bw,eval.points=vettore)
      AMLE[ss] = vettore[which.max(f_eps$estimate)]
  }
  }
  res = list(HGM = HGMEst, ABCSample = ABCsam, AMLE = AMLE)
}
res
}